using Microsoft.AspNetCore.Mvc;
using SchoolManagementSidekick;
using SchoolManagementSidekick.Filters;
using System;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace SchoolManagementSidekick
{
    [ApiController]
    [EntityValidation]
    [Route("api/[controller]/[action]")]
    [ResponseWrapperFilter]
    public class PersonController : ControllerBase
    {
        private readonly IPersonService _service;

        public PersonController(IPersonService service)
        {
            _service = service;
        }

        [HttpGet]
        public Task<IEnumerable<Person>> FilterByFirstName([FromQuery] string firstName)
        {
            return _service.FilterByFirstName(firstName, HttpContext.RequestAborted);
        }

        [HttpGet]
        public Task<int?> GetHighestGradedCourseForStudent([FromQuery] int studentId)
        {
            return _service.GetHighestGradedCourseForStudent(studentId);
        }

        [HttpPost]
        public Task<decimal> InsertPersonAsync(
            [FromQuery] string? lastName, [FromQuery] string? firstName, [FromQuery] DateTime? hireDate, [FromQuery] DateTime? enrollmentDate, [FromQuery] string? discriminator)
        {
            return _service.InsertPersonAsync(lastName, firstName, hireDate, enrollmentDate, discriminator, HttpContext.RequestAborted);
        }

        [HttpGet]
        public Task<Person?> GetAsync([FromQuery] int key)
        {
            return _service.GetAsync(key, HttpContext.RequestAborted);
        }

        [HttpGet]
        public Task<IEnumerable<Person>> GetListAsync()
        {
            return _service.GetListAsync(HttpContext.RequestAborted);
        }

        [HttpGet]
        public Task<PagedResult<Person>> GetPageAsync([FromQuery] PageParameter parameter)
        {
            return _service.GetPageAsync(parameter, HttpContext.RequestAborted);
        }

        [HttpGet]
        public Task<int> CountAsync()
        {
            return _service.CountAsync(HttpContext.RequestAborted);
        }
    }
}
