using Microsoft.Data.SqlClient;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Data;
using System.Threading;
using System.Threading.Tasks;

namespace SchoolManagementSidekick
{

    public partial class DataContext
    {
        private IDataContextProcedures _procedures;

        public virtual IDataContextProcedures Procedures
        {
            get
            {
                if (_procedures is null) 
                {
                    _procedures = new DataContextProcedures(this);
                }
                return _procedures;
            }
            set
            {
                _procedures = value;
            }
        }

        public IDataContextProcedures GetProcedures()
        {
            return Procedures;
        }

        protected void OnModelCreatingGeneratedProcedures(ModelBuilder modelBuilder)
        {
            modelBuilder.Entity<DboInsertPersonResult>().HasNoKey().ToView(null);
        }
    }

    public partial class DataContextProcedures : IDataContextProcedures
    {
        private readonly DataContext _context;

        public DataContextProcedures(DataContext context)
        {
            _context = context;
        }


        public virtual async Task<List<DboInsertPersonResult>?> DboInsertPersonAsync(string? lastName, string? firstName, DateTime? hireDate, DateTime? enrollmentDate, string? discriminator, OutputParameter<int>? returnValue = null, CancellationToken cancellationToken = default)
        {
            var returnValueParameter = new SqlParameter
            {
                ParameterName = "returnValue",
                Direction = ParameterDirection.Output,
                SqlDbType = SqlDbType.Int
            };

            var sqlParameters = new []
            {
                new SqlParameter
                {
                    ParameterName = "LastName",
                    Size = 100,
                    Value = lastName ?? Convert.DBNull,
                    SqlDbType = SqlDbType.NVarChar
                },

                new SqlParameter
                {
                    ParameterName = "FirstName",
                    Size = 100,
                    Value = firstName ?? Convert.DBNull,
                    SqlDbType = SqlDbType.NVarChar
                },

                new SqlParameter
                {
                    ParameterName = "HireDate",
                    Scale = 3,
                    Value = hireDate ?? Convert.DBNull,
                    SqlDbType = SqlDbType.DateTime
                },

                new SqlParameter
                {
                    ParameterName = "EnrollmentDate",
                    Scale = 3,
                    Value = enrollmentDate ?? Convert.DBNull,
                    SqlDbType = SqlDbType.DateTime
                },

                new SqlParameter
                {
                    ParameterName = "Discriminator",
                    Size = 100,
                    Value = discriminator ?? Convert.DBNull,
                    SqlDbType = SqlDbType.NVarChar
                },

                returnValueParameter
            };

            var _ = await _context.SqlQueryAsync<DboInsertPersonResult>("EXEC @returnValue = [dbo].[InsertPerson] @LastName = @LastName, @FirstName = @FirstName, @HireDate = @HireDate, @EnrollmentDate = @EnrollmentDate, @Discriminator = @Discriminator", sqlParameters, cancellationToken);

            returnValue?.SetValue(returnValueParameter.Value);

            return _;
        }

    }

    public class OutputParameter<TValue>
    {
        private bool _valueSet = false;

        public TValue? _value;
        public TValue? Value
        {
            get
            {
                if (!_valueSet)
                    throw new InvalidOperationException("Value not set.");

                return _value;
            }
        }

        internal void SetValue(object? value)
        {
            _valueSet = true;
            _value = null == value || Convert.IsDBNull(value) ? default : (TValue)value;
        }
    }
}