﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Filters;

namespace SchoolManagementSidekick.Filters
{
    [AttributeUsage(AttributeTargets.Class, AllowMultiple = false)]
    public class ResponseWrapperFilterAttribute : ActionFilterAttribute
    {
        public override void OnResultExecuting(ResultExecutingContext context)
        {
            if (context.Result is ObjectResult objectResult)
            {
                var wrappedResponse = new
                {
                    Data = objectResult.Value,
                    Timestamp = DateTime.UtcNow,
                    Status = objectResult.StatusCode ?? 200,
                    Success = (objectResult.StatusCode ?? 200) is >= 200 and < 300,
                };

                context.Result = new ObjectResult(wrappedResponse)
                {
                    StatusCode = objectResult.StatusCode
                };
            }


            base.OnResultExecuting(context);
        }
    }
}
