using Microsoft.EntityFrameworkCore;
using SchoolManagementSidekick;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq.Expressions;
using System.Threading;
using System.Threading.Tasks;

namespace SchoolManagementSidekick
{
    public class PersonService : EntityServiceBase<Person, int>, IPersonService
    {
        private readonly ICurrentUser _currentUser;
        private readonly DataContext _dataContext;

        public PersonService(IUnitOfWork unitOfWork, ICurrentUser currentUser, DataContext dataContext) : base(unitOfWork)
        {
            _currentUser = currentUser;
            _dataContext = dataContext;
        }

        public async Task<decimal> InsertPersonAsync(
            string? lastName,
            string? firstName,
            DateTime? hireDate,
            DateTime? enrollmentDate,
            string? discriminator,
            CancellationToken ct)
        {
            OutputParameter<int> outputParameter = new();
            var result = await _dataContext.Procedures.DboInsertPersonAsync(
                lastName,
                firstName,
                hireDate,
                enrollmentDate,
                discriminator,
                outputParameter,
                ct);
            if (result is not null)
            {
                return result
                    .First()
                    .NewPersonId ?? throw new InvalidOperationException("Error occurred when inserting entity");
            }

            throw new InvalidOperationException("Unknown occurred when inserting entity");

        }

        public IQueryable<TEntity> GetFiltered<TEntity>(Expression<Func<TEntity, bool>> filter, CancellationToken token = default)
            where TEntity : class
        {
            return _unitOfWork.GetRepository<TEntity>().GetQueryable(false)
                .Where(filter);
        }

        public async Task<IEnumerable<Person>> FilterByFirstName(string firstName, CancellationToken token = default)
        {
            return await GetFiltered<Person>(p => p.FirstName.Contains(firstName), token).ToListAsync();
        }

        public async Task<int?> GetHighestGradedCourseForStudent(int studentId)
        {
            return await (from user in GetFiltered<Person>(p => p.PersonId == studentId)
                          where user.Discriminator == "Student"
                          select DataContext.DboUfGetHighestGradedCourse(user.PersonId))
                   .FirstOrDefaultAsync();
        }



    }
}
